Package: BiocManager
Title: Access the Bioconductor Project Package Repository
Description: A convenient tool to install and update Bioconductor packages.
Version: 1.30.27
Authors@R: c(
    person("Martin", "Morgan", email = "martin.morgan@roswellpark.org",
        role = "aut", comment = c(ORCID = "0000-0002-5874-8148")),
    person("Marcel", "Ramos", email = "marcel.ramos@sph.cuny.edu",
        role = c("aut", "cre"), comment = c(ORCID = "0000-0002-3242-0582")))
Imports: utils
Suggests: BiocVersion, BiocStyle, remotes, rmarkdown, testthat, withr,
        curl, knitr
URL: https://bioconductor.github.io/BiocManager/
BugReports: https://github.com/Bioconductor/BiocManager/issues
VignetteBuilder: knitr
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-11-13 19:48:09 UTC; mramos
Author: Martin Morgan [aut] (ORCID: <https://orcid.org/0000-0002-5874-8148>),
  Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>)
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>
Repository: CRAN
Date/Publication: 2025-11-14 08:30:02 UTC
Built: R 4.6.0; ; 2026-04-25 15:56:04 UTC; unix
