Package: BiocManager
Title: Access the Bioconductor Project Package Repository
Description: A convenient tool to install and update Bioconductor packages.
Version: 1.30.17
Authors@R: c(
    person("Martin", "Morgan", email = "martin.morgan@roswellpark.org",
        role = "aut", comment = c(ORCID = "0000-0002-5874-8148")),
    person("Marcel", "Ramos", email = "marcel.ramos@roswellpark.org",
        role = c("ctb", "cre"), comment = c(ORCID = "0000-0002-3242-0582")))
Imports: utils
Suggests: BiocVersion, remotes, rmarkdown, testthat, withr, curl, knitr
BugReports: https://github.com/Bioconductor/BiocManager/issues
VignetteBuilder: knitr
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.1.2
NeedsCompilation: no
Packaged: 2022-04-21 22:43:48 UTC; mramos
Author: Martin Morgan [aut] (<https://orcid.org/0000-0002-5874-8148>),
  Marcel Ramos [ctb, cre] (<https://orcid.org/0000-0002-3242-0582>)
Maintainer: Marcel Ramos <marcel.ramos@roswellpark.org>
Repository: CRAN
Date/Publication: 2022-04-22 00:20:02 UTC
